"use strict";

import * as GeneralController from "../../Global/GeneralController.js";
import * as TileBorderManager from "../../Util/TileBorderManager.js";
import * as StageContainer    from "../../Stages/StageContainer.js";
import * as ScoreManager      from "../../Util/ScoreManager.js";
import * as GlobalData        from "../../Global/GlobalData.js";
import * as GameField         from "../../Global/GameField.js";
import * as Tools             from "../../Util/Tools.js";
import * as View              from "../../Util/View.js";

import { class_RepetitionChecker } from "../../Util/Class_RepetitionChecker.js";
import { class_BlinkingText }      from "../../Util/Class_BlinkingText.js";

const textBlinking_01 = new class_BlinkingText("Stage cleared !", "30px Verdana", 500, 160);

const repetitionChecker = new class_RepetitionChecker();

let stage;

let hasScoreImproved   = false;
let hasHiScoreImproved = false;

export function init()
{
	updateScores();
	textBlinking_01.reset();
}

function updateScores()
{
	stage = StageContainer.getCurrentStage();
	stage.sessionRemaining  = Tools.countRemainingTiles();

	hasScoreImproved   = (stage.finalScore > 0 && stage.sessionScore > stage.finalScore);
	hasHiScoreImproved = (stage.sessionHiScore > stage.finalHiScore);

	stage.finalScore     = stage.sessionScore;
	stage.finalHiScore   = stage.sessionHiScore;
	stage.finalRemaining = stage.sessionRemaining;

	localStorage.setItem(`${ stage.getIndex() }_score`,     stage.finalScore);
	localStorage.setItem(`${ stage.getIndex() }_hiscore`,   stage.finalHiScore);
	localStorage.setItem(`${ stage.getIndex() }_remaining`, stage.finalRemaining);
}

export function handleGameTick()
{
	handleInput();
	draw();
}

function handleInput()
{
	if (!repetitionChecker.isRepeating(GeneralController.userInput))
	{
		switch (GeneralController.userInput)
		{
			case GlobalData.enumUserInput.TITLE_SCREEN : GeneralController.switchToTitleScreen() ; break;
			case GlobalData.enumUserInput.PLAY_SCREEN  : GeneralController.switchToPlayScreen()  ; break;
		}
	}
}

function draw()
{
	View.context.clearRect(0, 0, 500,350);
	GameField.render();
	TileBorderManager.render();
	ScoreManager.render();
	textBlinking_01.handleGameTick();

	let text = "";
	let X = 0;
	let Y = 300;
	
	if (hasHiScoreImproved)
	{
		View.context.fillStyle = "yellow";
		
		text = "excellent !";
		View.context.font = "24px Verdana";
		X = Tools.calculateTextXcoordinate(text);
		View.context.fillText(text, X, Y);

		Y += 50;
		text = "you reached a new hi-score !";
		View.context.font = "32px Verdana";
		X = Tools.calculateTextXcoordinate(text);
		View.context.fillText(text, X, Y);
	}
	else 	if (hasScoreImproved)
	{
		View.context.fillStyle = "cyan";
		
		text = "well done !";		
		View.context.font = "22px Verdana";
		X = Tools.calculateTextXcoordinate(text);
		View.context.fillText(text, X, Y);

		Y += 50;
		text = "you improved your score !";
		View.context.font = "28px Verdana";
		X = Tools.calculateTextXcoordinate(text);
		View.context.fillText(text, X, Y);
	}

	View.context.font = "20px Verdana";
	View.context.fillStyle = "gray";
	Y = 825;

	text = "press F3 to play this stage again";
	X = Tools.calculateTextXcoordinate(text);
	View.context.fillText(text, X, Y);

	Y += 30;
	text = "press F1 to select another stage";
	X = Tools.calculateTextXcoordinate(text);
	View.context.fillText(text, X, Y);
}
