"use strict";

import * as GeneralController from "../../Global/GeneralController.js";
import * as TileBorderManager from "../../Util/TileBorderManager.js";
import * as ExplosionManager  from "../../Util/Explosion/ExplosionManager.js";
import * as TextPopupManager  from "./../../Util/TextPopup/TextPopupManager.js";
import * as StageContainer    from "../../Stages/StageContainer.js";
import * as GameController    from "./GameController.js";
import * as GravityManager    from "../../Util/GravityManager.js";
import * as ScoreManager      from "../../Util/ScoreManager.js";
import * as UndoManager       from "../../Util/UndoManager.js";
import * as TileFinder        from "../../Util/TileFinder.js";
import * as GlobalData        from "../../Global/GlobalData.js";
import * as GameField         from "../../Global/GameField.js";
import * as Tools             from "../../Util/Tools.js";
import * as View              from "../../Util/View.js";

import { class_Delay }            from "../../Util/Class_Delay.js";
import { class_ExplosionCluster } from "../../Util/Explosion/Class_ExplosionCluster.js";

const delayTransition  = new class_Delay(75);

let tileLocationsSet = new Set();

let screenState;
const enumScreenState = Object.freeze({
	TRANSITION : 10,
	NORMAL     : 11
});

let transitionStep;
const TRANSITION_STEP_MAX = 10;
const TRANSITION_STEP_MIN = 0;

export function init()
{
	screenState = enumScreenState.TRANSITION;
	transitionStep = TRANSITION_STEP_MAX;
	StageContainer.resetScore();
}

export function handleGameTick()
{
	if (screenState === enumScreenState.TRANSITION)
	{
		render();
		triggerTransition();
	}
	else if (screenState === enumScreenState.NORMAL)
	{
		mainGameLoop();
		render();
	}
}

function mainGameLoop()
{
	// Check of er op een tile geklikt is :
	const row = GeneralController.getSelectedRowAndCol()[0];
	const col = GeneralController.getSelectedRowAndCol()[1];
	GeneralController.resetMouseClick();

	// Als er op een tile geklikt is, verzamel dan alle omliggende, identieke tiles :
	tileLocationsSet.clear();
	if (row > GeneralController.NO_SELECTION && col > GeneralController.NO_SELECTION)
	{
		tileLocationsSet = TileFinder.search(row, col);
	}

	// Zet de gevonden tiles om in explosies :
	if (tileLocationsSet.size > 1)
	{
		const explosionCluster = new class_ExplosionCluster(row, col, tileLocationsSet);
		ExplosionManager.addExplosionCluster(explosionCluster);
	}

	// Als er explosies klaar zijn, zorg dan dat de ontstane lege plekken worden opgevuld :
	const explosionsFinished = ExplosionManager.updateState();
	if (explosionsFinished)
	{
		GravityManager.applyGravity();

		// Als de GravityManager ook [explosies] verplaatst heeft,
		// dan moeten die explosies hun nieuwe locatie weten :
		const newExplosionLocationsMap = GameField.getExplosionLocationsMap();
		ExplosionManager.updateExplosionWithNewLocations(newExplosionLocationsMap);

		UndoManager.addUndoData();

		// Check of er geen explosies bezig zijn en of er nog steeds klikbare tile-combi's zijn :
		if (!ExplosionManager.hasExplosions() && !Tools.areThereTileClustersLeft())
		{
			GameController.switchToGameOver();
		}
	}
}

function render()
{
	GameField.render();
	TileBorderManager.render();
	ExplosionManager.render();
	ScoreManager.render();
	TextPopupManager.handleGameTick();
}

//---

function triggerTransition()
{
	if (transitionStep > TRANSITION_STEP_MIN)
	{
		const HEIGHT = transitionStep * 5;
		drawBalkje(HEIGHT);
		if (delayTransition.hasExpired())
		{
			transitionStep--;
		}
	}
	else
	{
		transitionStep = TRANSITION_STEP_MAX;
		screenState = enumScreenState.NORMAL;
	}
}

function drawBalkje(height)
{
	const X = Tools.getTileX(0);
	for (let row = 0; row < GlobalData.ROW_COUNT; row++)
	{
		const Y = Tools.getTileY(row);

		View.context.fillStyle = "orange";
		View.context.fillRect(X, Y, GlobalData.CANVAS_WIDTH, height);
	}
}
