"use strict";

import * as StageContainer  from "../../Stages/StageContainer.js";
import * as TitleScreen_01  from "./TitleScreen_01.js";
import * as GlobalData      from "../../Global/GlobalData.js";
import * as Tools           from "../../Util/Tools.js";
import * as View            from "../../Util/View.js";

import { class_Delay }   from "../../Util/Class_Delay.js";
import { class_Point }   from "../../Util/Class_Point.js";

export class class_TextRow
{
	#delaySelectionBar = new class_Delay(500);

	#index;

	#xBase = 320;
	#yBase = 350;

	#OFFSET_VERTICAL = 65;

	#X; #Y;

	#toggleAnimation = true;

	boundingBoxStartPoint = new class_Point(0,0);
	boundingBoxEndPoint   = new class_Point(0,0);

	constructor(index)
	{
		this.#index = index;
		this.#initCoordinates();
		this.#initBoundingPoints();
		this.#initScores();
	}

	#initCoordinates()
	{
		this.#X = this.#xBase;
		this.#Y = this.#yBase + (this.#index * this.#OFFSET_VERTICAL);
	}

	#initBoundingPoints()
	{
		let X = this.#X - 10;
		let Y = this.#Y - 35;
		this.boundingBoxStartPoint.X = X;
		this.boundingBoxStartPoint.Y = Y;

		X += 595;
		Y += 50;
		this.boundingBoxEndPoint.X = X;
		this.boundingBoxEndPoint.Y = Y;
	}

	#initScores()
	{
		const stage = StageContainer.stageMap.get(this.#index);

		const score = localStorage.getItem(`${ this.#index }_score`);
		stage.finalScore = (score !== null) ? score : 0;

		const hiScore = localStorage.getItem(`${ this.#index }_hiscore`);
		stage.finalHiScore  = (hiScore !== null) ? hiScore : GlobalData.HI_SCORE_PRESET;
		stage.sessionHiScore = stage.finalHiScore;

		const remaining = localStorage.getItem(`${ this.#index }_remaining`);
		stage.finalRemaining = (remaining !== null) ? remaining : 0;
	}

	draw()
	{
		let X = this.#X;
		let Y = this.#Y;

		View.context.lineWidth = 1.0;
		View.context.strokeStyle = "white";

		View.context.font = "bold 28px Verdana";
		// View.context.font = "35px fontGalacticStorm";
		View.context.fillStyle = "yellow";
		View.context.fillText(StageContainer.stageMap.get(this.#index).name, X, Y);

		View.context.font = "bold 24px Verdana";
		// View.context.font = "30px fontGalacticStorm";
		View.context.fillStyle = "orange";

		const stage = StageContainer.stageMap.get(this.#index);

		X += 200;
		View.context.fillText(String(stage.finalScore).padStart(GlobalData.MAX_SCORE_LENGTH, '0'), X, Y);

		X += 180;
		View.context.fillText(String(stage.finalHiScore).padStart(GlobalData.MAX_SCORE_LENGTH, '0'), X, Y);

		X += 180;
		View.context.fillText(stage.finalRemaining, X, Y);

		// Draw bounding-box als debug-info.
		// Dit is de rechthoek waarin geklikt moet worden om een bepaalde Stage te selecteren.
		// {
		// 	let X1 = this.boundingBoxStartPoint.X;
		// 	let Y1 = this.boundingBoxStartPoint.Y;
		//
		// 	let X2 = this.boundingBoxEndPoint.X;
		// 	let Y2 = this.boundingBoxEndPoint.Y;
		//
		// 	View.context.strokeStyle = "white";
		//
		// 	View.context.beginPath();
		// 	View.context.moveTo(X1,Y1);
		// 	View.context.lineTo(X2,Y2);
		// 	View.context.closePath();
		// 	View.context.stroke();
		//
		// 	View.context.beginPath();
		// 	View.context.moveTo(X1,Y1);
		// 	View.context.lineTo(X2,Y1);
		// 	View.context.lineTo(X2,Y2);
		// 	View.context.lineTo(X1,Y2);
		// 	View.context.closePath();
		// 	View.context.stroke();
		// }

		if (this.#index === TitleScreen_01.selectedStageIndex)
		{
			X = this.boundingBoxStartPoint.X;
			Y = this.boundingBoxStartPoint.Y + 23;
			this.#drawSelectionBarWithCoords(X, Y);
		}
	}

	#drawSelectionBarWithCoords(X, Y)
	{
		if (this.#delaySelectionBar.hasExpired())
		{
			this.#toggleAnimation = !this.#toggleAnimation;
		}

		if (this.#toggleAnimation)
		{
			View.context.setLineDash([15, 5]);
		}
		else
		{
			View.context.setLineDash([10, 5]);
		}

		Tools.drawOvalStroked(X, Y, 605, 28);
		View.context.setLineDash([0]); // Restore default-value.
	}

}
