"use strict";

import * as TitleScreenController from "../GeneralStates/TitleScreen/TitleScreenController.js";
import * as HelpScreenController  from "../GeneralStates/HelpScreen/HelpScreenController.js";
import * as GameController        from "../GeneralStates/PlayScreen/GameController.js";
import * as UndoManager           from "../Util/UndoManager.js";
import * as GlobalData            from "./GlobalData.js";
import * as GameField             from "./GameField.js";

import { class_RepetitionChecker } from "../Util/Class_RepetitionChecker.js";

const enumGeneralState = Object.freeze({
	TITLE_SCREEN : 10,
	HELP_SCREEN  : 11,
	PLAY_SCREEN  : 12
});

const repetitionChecker = new class_RepetitionChecker();

let currentState = enumGeneralState.PLAY_SCREEN;
export let userInput = GlobalData.enumUserInput.NONE;

export const NO_SELECTION = -1;

const mouseClicked_Row_and_Col = [];
const mouseClicked_X_and_Y = [];

export function setUserInput(input)
{
	userInput = input;
}

export function setClickedLocation(X, Y)
{
	mouseClicked_X_and_Y.length = 0;
	mouseClicked_X_and_Y.push(X);
	mouseClicked_X_and_Y.push(Y);

	const col = Math.trunc((X - GlobalData.OFFSET_GAME_FIELD_LEFT) / GlobalData.TILE_SIZE);
	const row = Math.trunc((Y - GlobalData.OFFSET_GAME_FIELD_TOP)  / GlobalData.TILE_SIZE);
	setSelectedColAndRow(row, col);
}

function resetClickedLocation()
{
	mouseClicked_X_and_Y.length = 0;
	mouseClicked_X_and_Y.push(NO_SELECTION);
	mouseClicked_X_and_Y.push(NO_SELECTION);
}

export function getClickedLocation()
{
	return mouseClicked_X_and_Y;
}

function setSelectedColAndRow(row, col)
{
	mouseClicked_Row_and_Col.length = 0;
	mouseClicked_Row_and_Col.push(row);
	mouseClicked_Row_and_Col.push(col);
}

function resetSelectedColAndRow()
{
	mouseClicked_Row_and_Col.length = 0;
	mouseClicked_Row_and_Col.push(NO_SELECTION);
	mouseClicked_Row_and_Col.push(NO_SELECTION);
}

export function resetMouseClick()
{
	resetClickedLocation();
	resetSelectedColAndRow();
}

export function getSelectedRowAndCol()
{
	return mouseClicked_Row_and_Col;
}

export function handleGameTick()
{
	handleInput();
	distributeGameTick();
}

function handleInput()
{
	if (!repetitionChecker.isRepeating(userInput))
	{
		switch (userInput)
		{
			case GlobalData.enumUserInput.TITLE_SCREEN : switchToTitleScreen() ; break;
			case GlobalData.enumUserInput.PLAY_SCREEN  : switchToPlayScreen()  ; break;
			case GlobalData.enumUserInput.HELP_SCREEN  : switchToHelpScreen()  ; break;
		}
	}
}

export function switchToTitleScreen()
{
	resetMouseClick(); // Negeer premature mouse-clicks.
	TitleScreenController.init();
	currentState = enumGeneralState.TITLE_SCREEN;
}

export function switchToHelpScreen()
{
	resetMouseClick(); // Negeer premature mouse-clicks.
	currentState = enumGeneralState.HELP_SCREEN;
}

export function switchToPlayScreen()
{
	resetMouseClick(); // Negeer premature mouse-clicks.
	GameController.init();
	UndoManager.reset();
	GameField.addTilesToGameField();
	currentState = enumGeneralState.PLAY_SCREEN;
}

function distributeGameTick()
{
	switch (currentState)
	{
		case enumGeneralState.TITLE_SCREEN : TitleScreenController.handleGameTick() ; break;
		case enumGeneralState.HELP_SCREEN  : HelpScreenController.handleGameTick()  ; break;
		case enumGeneralState.PLAY_SCREEN  : GameController.handleGameTick()        ; break;
	}
}
