"use strict";

import * as GlobalData from "../Global/GlobalData.js";
import * as GameField  from "../Global/GameField.js";
import * as Tools      from "../Util/Tools.js";

let cols = [];
let tileColors = [];

export function addCol(col)
{
	cols.push(col);
}

export function applyGravity()
{
	applyVerticalGravity();
	applyHorizontalGravity();
}

function applyVerticalGravity()
{
	while (cols.length > 0)
	{
		const col = cols.shift(); // 'shift' staat voor ; verwijder het [eerste] item, (aan het [begin] van de array, dus).
		collectTiles(col);
		resetCol(col);
		restoreCol(col);
	}
}

//-- Vertical.

function collectTiles(col)
{
	tileColors.length = 0;

	for (let row = 0 ; row < GlobalData.ROW_COUNT ; row++)
	{
		const tileColor = GameField.data[row][col];

		if (Tools.isValidTileOrExplosion(tileColor))
		{
			tileColors.push(tileColor);
		}
	}
}

function resetCol(col)
{
	for (let row = 0 ; row < GlobalData.ROW_COUNT ; row++)
	{
		GameField.setTile(row, col, GlobalData.enumTileNumber.NO_TILE);
	}
}

function restoreCol(col)
{
	let row = GlobalData.ROW_MAX;
	while (tileColors.length > 0)
	{
		GameField.setTile(row, col, tileColors.pop());
		row--;
	}
}

//-- Horizontal.

function applyHorizontalGravity()
{
	const emptyColumnsArray = selectEmptyColumns();
	if (emptyColumnsArray.length > 0)
	{
		const arrayRows = selectTilesToMove(emptyColumnsArray);
		replaceSelectedTiles(arrayRows);
	}
}

function selectEmptyColumns()
{
	// Check of er ergens een lege plek zit in de onderste rij.
	// Zo ja, dan moeten alle rechtsgelegen kolommen een stap
	// naar links worden opgeschoven.

	// Deze plek moet dan echt *leeg* zijn en [geen] explosie,
	// want na een explosie moet eerst de gravity zijn werk hebben
	// gedaan voordat er gecheckt kan worden op lege kolommen.

	const emptyColumnsArray = [];
	for (let col = 0 ; col < GlobalData.COL_COUNT ; col++)
	{
		const tileColor = GameField.data[GlobalData.ROW_MAX][col];
		if (Tools.isEmptyTile(tileColor))
		{
			emptyColumnsArray.push(col);
		}
	}
	return emptyColumnsArray;
}

function selectTilesToMove(emptyColumnsArray)
{
	const arrayRows = [];

	for (let row = GlobalData.ROW_MAX ; row >= GlobalData.ROW_MIN ; row--)
	{
		let validTilesFound = false;
		const arrayCols = [];

		for (let col = 0 ; col < GlobalData.COL_COUNT ; col++)
		{
			const tileColor = GameField.data[row][col];
			if (Tools.isValidTileOrExplosion(tileColor))
			{
				validTilesFound = true;
				arrayCols.push(tileColor);
			}
			else if (!emptyColumnsArray.includes(col))
			{
				arrayCols.push(GlobalData.enumTileNumber.NO_TILE);
			}
		}

		if (arrayCols.length > 0 && validTilesFound)
		{
			arrayRows.unshift(arrayCols); // 'unshift' staat voor ; voeg dit item toe aan het [begin] van de array.
		}

		if (!validTilesFound)
		{
			break;
		}
	}

	return arrayRows;
}

function replaceSelectedTiles(arrayRows)
{
	let row = GlobalData.ROW_MAX;
	while (arrayRows.length > 0)
	{
		GameField.data[row].fill(GlobalData.enumTileNumber.NO_TILE);

		const tiles = arrayRows.pop();
		for (let col = 0 ; col < tiles.length ; col++)
		{
			GameField.setTile(row, col, tiles[col]);
		}

		row--;
	}
}
